@@ -102,6 +102,10 @@ checks in boot time. By default is set to `true` allowing checks on boot.
 ext2, ext3, ext4 and xfs.
 
 - partuuid -- GPT partition UUID string.
+A version 5 UUID can be easily generated using the uuid5 template function
+{{ uuid5 $namespace $data }} $namespace should be a valid UUID and $data can be
+any string, to generate reproducible UUID value pass a fixed value of namespace
+and data.
 
 - extendedoptions -- list of additional filesystem extended options which need
 to be enabled for the partition.
@@ -338,8 +342,21 @@ func (i ImagePartitionAction) formatPartition(p *Partition, context debos.DebosC
 
 	cmdline := []string{}
 	switch p.FS {
-	case "vfat":
-		cmdline = append(cmdline, "mkfs.vfat", "-F32", "-n", p.Name)
+	case "fat", "fat12", "fat16", "fat32", "msdos", "vfat":
+		cmdline = append(cmdline, "mkfs.vfat", "-n", p.Name)
+
+		switch p.FS {
+		case "fat12":
+			cmdline = append(cmdline, "-F12")
+		case "fat16":
+			cmdline = append(cmdline, "-F16")
+		case "fat32", "msdos", "vfat":
+			cmdline = append(cmdline, "-F32")
+		default:
+			/* let mkfs.vfat autodetermine FAT type */
+			break
+		}
+
 		if len(p.FSUUID) > 0 {
 			cmdline = append(cmdline, "-i", p.FSUUID)
 		}
@@ -444,8 +461,6 @@ func (i *ImagePartitionAction) PreNoMachine(context *debos.DebosContext) error {
 }
 
 func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
-	i.LogStart()
-
 	/* On certain disk device events udev will call the BLKRRPART ioctl to
 	 * re-read the partition table. This will cause the partition devices
 	 * (e.g. vda3) to temporarily disappear while the rescanning happens.
@@ -478,15 +493,29 @@ func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
 		}
 
 		var name string
-		if i.PartitionType == "gpt" {
-			name = p.PartLabel
+		if i.PartitionType == "msdos" {
+			if len(i.Partitions) <= 4 {
+				name = "primary"
+			} else {
+				if idx < 3 {
+					name = "primary"
+				} else if idx == 3 {
+					name = "extended"
+				} else {
+					name = "logical"
+				}
+			}
 		} else {
-			name = "primary"
+			name = p.PartLabel
 		}
 
 		command := []string{"parted", "-a", "none", "-s", "--", context.Image, "mkpart", name}
 		switch p.FS {
-		case "vfat":
+		case "fat16":
+			command = append(command, "fat16")
+		case "fat", "fat12", "fat32", "msdos", "vfat":
+			/* TODO: Not sure if this is correct. Perhaps
+			   fat12 should be treated the same as fat16 ? */
 			command = append(command, "fat32")
 		case "hfsplus":
 			command = append(command, "hfs+")
@@ -658,6 +687,37 @@ func (i ImagePartitionAction) PostMachineCleanup(context *debos.DebosContext) er
 }
 
 func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
+
+	if i.PartitionType == "msdos" {
+		for idx, _ := range i.Partitions {
+			p := &i.Partitions[idx]
+
+			if idx == 3 && len(i.Partitions) > 4 {
+				var name string
+				var part Partition
+
+				name = "extended"
+				part.number = idx + 1
+				part.Name = name
+				part.Start = p.Start
+				tmp_n := len(i.Partitions) - 1
+				tmp := &i.Partitions[tmp_n]
+				part.End = tmp.End
+				part.FS = "none"
+
+				i.Partitions = append(i.Partitions[:idx+1], i.Partitions[idx:]...)
+				i.Partitions[idx] = part
+
+				num := 1
+				for idx, _ := range i.Partitions {
+					p := &i.Partitions[idx]
+					p.number = num
+					num++
+				}
+			}
+		}
+	}
+
 	if len(i.GptGap) > 0 {
 		log.Println("WARNING: special version of parted is needed for 'gpt_gap' option")
 		if i.PartitionType != "gpt" {
@@ -710,7 +770,7 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 				if err != nil {
 					return fmt.Errorf("Incorrect UUID %s", p.FSUUID)
 				}
-			case "vfat", "fat32":
+			case "fat", "fat12", "fat16", "fat32", "msdos", "vfat":
 				_, err := hex.DecodeString(p.FSUUID)
 				if err != nil || len(p.FSUUID) != 8 {
 					return fmt.Errorf("Incorrect UUID %s, should be 32-bit hexadecimal number", p.FSUUID)
@@ -756,10 +816,7 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 			return fmt.Errorf("Partition %s missing end", p.Name)
 		}
 
-		switch p.FS {
-		case "fat32":
-			p.FS = "vfat"
-		case "":
+		if p.FS == "" {
 			return fmt.Errorf("Partition %s missing fs type", p.Name)
 		}
 	}
@@ -784,6 +841,10 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 		if m.part == nil {
 			return fmt.Errorf("Couldn't find partition for %s", m.Mountpoint)
 		}
+
+		if strings.ToLower(m.part.FS) == "none" {
+			return fmt.Errorf("Cannot mount %s: filesystem not present", m.Mountpoint)
+		}
 	}
 
 	// Calculate the size based on the unit (binary or decimal)
