@@ -30,10 +30,12 @@ Comments are allowed and should be prefixed with '#' symbol.
      property2: {{$Var}}
 
 
-The following custom template functions are available
+The following custom template functions are available:
 
 - sector: Returns the argument * 512 (convential sector size) e.g. `{{ sector 64 }}`
 - escape: Shell escape the  argument `{{ escape $var }}`
+- uuid5: Generates fixed UUID value `{{ uuid5 $random-uuid $text }}`
+- functions from [slim-sprig](https://go-task.github.io/slim-sprig/)
 
 Mandatory properties for recipe:
 
@@ -81,11 +83,13 @@ import (
 	"github.com/go-debos/debos"
 	"gopkg.in/yaml.v2"
 	"github.com/alessio/shellescape"
+	"github.com/go-task/slim-sprig/v3"
 	"path"
 	"text/template"
 	"log"
 	"strings"
 	"reflect"
+	"github.com/google/uuid"
 )
 
 /* the YamlAction just embed the Action interface and implements the
@@ -156,6 +160,11 @@ func escape(s string) string {
 	return shellescape.Quote(s)
 }
 
+func uuid5(namespace string, data string) string {
+	id := uuid.NewSHA1(uuid.MustParse(namespace), []byte(data))
+	return id.String()
+}
+
 func DumpActionStruct(iface interface{}) string {
 	var a []string
 
@@ -243,9 +252,13 @@ func (r *Recipe) Parse(file string, printRecipe bool, dump bool, templateVars ..
 	funcs := template.FuncMap{
 		"sector": sector,
 		"escape": escape,
+		"uuid5": uuid5,
 	}
 	t.Funcs(funcs)
 
+	/* Add slim-sprig functions to template language */
+	t.Funcs(sprig.FuncMap())
+
 	if _, err := t.ParseFiles(file); err != nil {
 		return err
 	}
